-- Test of switching between sprite and outline.
-- Test of use of stencils.
-- [Test of dropping an image onto a prexisting window.]

local gr			= love.graphics
local girl		= require 'girl'
local stencil	= require 'stencil'

function love.load()
	screen			= { w = 500, h = 400 }
	love.window.setMode(screen.w,screen.h,{borderless=true})
	love.keyboard.setKeyRepeat( true )
	girl.mx			= screen.w
	stencil.area	= { x=259, y=200, w=150, h=60, sides=20 }
	stencil.poly	= stencil.create( stencil.area )
	--[[	-- alternative optional shapes or any polygon shape
	params = {250,200,50}								-- x,y,radius
	stencil.shape( gr.circle, params )	-- could use shape( func, ... ) but parms lost
	params = {200,160,100,80}							-- x,y,w,h[,rx,ry]
	stencil.shape( gr.rectangle, params )
	params = stencil.newPoly(250,200,150,60,20)	-- x,y,w,h,sides
	stencil.shape( gr.polygon, params )				-- actually vertices - {x1,y1,x2,y2...}
	params = stencil.newEllipse(250,200,30,70,-math.pi/8)	-- x,y,rx,ry[,an]
	stencil.shape( gr.polygon, params )
	params = stencil.newRectangle(50,50,50,80,math.pi/4)	-- x,y,w,h[,an]
	stencil.shape( gr.polygon, params )
	]]
	end

function love.update( dt )
	girl.update( dt )
	end

function love.draw()
	gr.setColor( 1, 1, 1 )
	stencil.draw( 'outside', girl.draw, 1 )	-- where drawn, drawing func, params
	stencil.draw( 'inside',  girl.draw, 2 )

	if showBoundingBox then
		gr.setColor( 1, 1, 1 )
		stencil.draw('outside',gr.rectangle,'line', girl.x-girl.sz*girl.w/2, girl.y-girl.sz*girl.h/2, girl.sz*girl.w, girl.sz*girl.h )
		gr.setColor( 255, 0, 0 )
		stencil.draw('inside',gr.rectangle,'line', girl.x-girl.sz*girl.w/2, girl.y-girl.sz*girl.h/2, girl.sz*girl.w, girl.sz*girl.h )
		end

	gr.setColor( 255, 255, 0 )
	gr.print( 'fps: '..love.timer.getFPS(), 10, 10 )
	gr.print( 'size:wsad\npos:arrows\nsides:cv\ngirl size:zx\nbounds:b', 10, 320)
	gr.rectangle('line',1,1,screen.w-2,screen.h-2)	-- window frame
	gr.polygon( 'line', stencil.poly )					-- show stencil area
	--gr.polygon('line',unpack(params))
	--gr.circle('line',unpack(params))
	--gr.rectangle('line',unpack(params))
	--[[	unrelated
	if newImage then
		gr.setColor( 255, 255, 255, 255 )
		gr.draw( newImage, screen.w/2, screen.h/2, 0, 1, 1,newImage:getWidth()/2, newImage:getHeight()/2 )
		end
		]]
	end

function love.keypressed( key )
	if key == 'escape'	then love.event.quit() end
	if key == 'b'			then showBoundingBox = not showBoundingBox end
	if key == 'x'			then girl.sz = girl.sz + .1 end
	if key == 'z'			then girl.sz = girl.sz - .1 end
	if key == 'w'			then stencil.area.h = stencil.area.h+1 end
	if key == 's'			then stencil.area.h = stencil.area.h-1 end
	if key == 'a'			then stencil.area.w = stencil.area.w-1 end
	if key == 'd'			then stencil.area.w = stencil.area.w+1 end
	if key == 'up'			then stencil.area.y = stencil.area.y-1 end
	if key == 'down'		then stencil.area.y = stencil.area.y+1 end
	if key == 'left'		then stencil.area.x = stencil.area.x-1 end
	if key == 'right'		then stencil.area.x = stencil.area.x+1 end
	if key == 'c'			then stencil.area.sides = stencil.area.sides>3 and stencil.area.sides-1 or 3 end
	if key == 'v'			then stencil.area.sides = stencil.area.sides+1 end
	stencil.poly = stencil.create( stencil.area )
	end
--[[ completely unrelated
function love.filedropped( file )
	local fullpath				= file:getFilename()
	local pattern				= '(.-)([^\\/]-%.?([^%.\\/]*))$'
	local path, name, ext	= fullpath:match( pattern )
	if ext == 'png' or ext == 'jpg' then
		print( 'Image file '..name..' loaded from '..path )
		assert( file:open( 'r' ) )
			local contents		= file:read()
			file:close()
		local data, err		= love.filesystem.newFileData( contents, name )
		local imageData		= love.image.newImageData( data )
		newImage					= gr.newImage( imageData )
		end
	end
	]]
